package core

import (
	"context"

	"github.com/anyproto/anytype-heart/pb"
)

func (mw *Middleware) LogSend(cctx context.Context, req *pb.RpcLogSendRequest) *pb.RpcLogSendResponse {
	response := func(code pb.RpcLogSendResponseErrorCode, err error) *pb.RpcLogSendResponse {
		m := &pb.RpcLogSendResponse{Error: &pb.RpcLogSendResponseError{Code: code}}
		if err != nil {
			m.Error.Description = getErrorDescription(err)
		}

		return m
	}

	switch req.Level {
	case pb.RpcLogSendRequest_FATAL:
		log.Fatal(req.Message)
	case pb.RpcLogSendRequest_PANIC:
		log.Panic(req.Message)
	case pb.RpcLogSendRequest_DEBUG:
		log.Debug(req.Message)
	case pb.RpcLogSendRequest_INFO:
		log.Info(req.Message)
	case pb.RpcLogSendRequest_WARNING:
		log.Warn(req.Message)
	case pb.RpcLogSendRequest_ERROR:
		log.Error(req.Message)
	}

	return response(pb.RpcLogSendResponseError_NULL, nil)
}
